//----------------------------------------------------------------------------
//  Project scenedit
//  Allgood Software
//  Copywrite 1995
//
//  SUBSYSTEM:    scenedit.apx Application
//  FILE:         proptreepage.h
//  AUTHOR:       Russell Morris
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for TPropTreePage (TPropertyPage).
//
//----------------------------------------------------------------------------
#if !defined(proptree_h)              // Sentry, use file only if it's not already included.
#define proptree_h

#include <owl/propsht.h>
#include <owl/treewind.h>
#include <owl/imagelst.h>
#include <owl/static.h>
#include <classlib/arrays.h>

#include "mymacro.h"

class TPropTreePane;

class SESSIONSEXPORT TPaneInfo {
  public:
    TPaneInfo();
    TPaneInfo( uint32 id );
    ~TPaneInfo();

    TPaneInfo& operator = (const TPaneInfo other);
    int operator == (const TPaneInfo other);

    // state info about pane
    //
    uint32  id;
    bool    dirty;
};

typedef TArrayAsVector< TPaneInfo > TPaneInfoArray;
typedef TArrayAsVectorIterator< TPaneInfo > TPaneInfoArrayIter;

class SESSIONSEXPORT TPropTreePage : public TPropertyPage {
  public:
    TPropTreePage( TPropertySheet* parent,
                   TResId resId,
                   const char far* title = 0,
                   TResId iconRes = 0,
                   TModule* module = 0);
    virtual ~TPropTreePage();

  public:
    virtual void SetupWindow();

    // Redefine TPropertyPage virtuals (nb. not present in TPropTreeDialog)
    //
    virtual   int   SetActive(TNotify&);    // PSN_SETACTIVE
    virtual   bool  KillActive(TNotify&);   // PSN_KILLACTIVE
    virtual   int   Apply(TNotify&);        // PSN_APPLY
    virtual   void  Reset(TNotify&);        // PSN_RESET
    virtual   void  Help(TNotify&);         // PSN_HELP
    virtual   int   WizBack(TNotify&);      // PSN_WIZBACK
    virtual   int   WizNext(TNotify&);      // PSN_WIZNEXT
    virtual   bool  WizFinish(TNotify&);    // PSN_FINISH
    virtual   bool  QueryCancel(TNotify&);  // PSN_QUERYCANCEL

    // Doxxx() is called by xxx(TNotify&) above.  Doxxx() handles
    // the notification for the page itself (validate controls, etc) while
    // xxx(TNotify&) runs through the TPropTreePane list, calling each
    // pane's xxx() function.
    //
    virtual   int   DoSetActive();
    virtual   bool  DoKillActive();
    virtual   int   DoApply();
    virtual   void  DoReset();
    virtual   void  DoHelp();
    virtual   int   DoWizBack();
    virtual   int   DoWizNext();
    virtual   bool  DoWizFinish();
    virtual   bool  DoQueryCancel();

    // Pane change notifications
    //
    virtual void PaneChanged( uint32 id );
    virtual void PaneUnchanged( uint32 id );

    // Add a pane to the tree
    //
    TTreeNode AddPane( TPropTreePane* pane,
                       TTreeNode parent,
                       const char far* text,
                       int i = 0,
                       int j = 0);

    // Pane intercommunication functions
    //
    TPropTreePane* GetPane( uint32 id );
    TPropTreePane* GetCurrentPane();

    // Wrap manipulation of nodes
    //
    void SelectPane( uint32 id, TTreeNode::TNextCode nc = TTreeNode::Caret );
    void SelectNextPane( TTreeNode::TNextCode nc);

    TPropTreePane* PaneFromId( uint32 id );
    TTreeNode NodeFromId( uint32 id );

    // Page notification
    //
    void PageChanged();
    void PageUnchanged();

  protected:
    // Respond to TreeView events
    //
    virtual bool  EvBeginLabelEdit( TTwDispInfoNotify& );
    virtual void  EvEndLabelEdit( TTwDispInfoNotify& );
    virtual void  EvGetDispInfo( TTwDispInfoNotify& );
    virtual void  EvItemExpanded( TTwNotify& );
    virtual bool  EvItemExpanding( TTwNotify& );
    virtual void  EvSelChanged( TTwNotify& );
    virtual bool  EvSelChanging( TTwNotify& );
    virtual void  EvSetDispInfo( TTwDispInfoNotify& );
    virtual void  EvKeyDown( TTwKeyDownNotify& );

    //
    // == These are all TTreeWindow functions ==
    // Use them to manipulate the underlying treeview control
    //
    // *nb: all drag-drop and delete functions are removed
    //

  public:
    TTreeNode GetRoot();
    TTreeNode GetSelection();
    TTreeNode GetFirstVisible();

    // Indent level
    //
    uint GetIndent();
    void SetIndent(uint);

    // Image list
    //
    HIMAGELIST GetImageList(TImageListType type);
    HIMAGELIST SetImageList(TImageListType type, HIMAGELIST newList);

    // Miscellaneous
    //
    uint GetPaneCount();
    uint GetVisibleCount();
    HWND GetEditControl();

    //
    //
    //
    void Update();
    bool EnsureVisible( uint32 paneId );
    bool ExpandPane(uint32 flag, uint32 paneId);
    HWND EditLabel( uint32 paneId );
    HTREEITEM HitTest(TTwHitTestInfo* info);
    HTREEITEM GetNextPane( uint32 nc, uint32 painId );

  private:
    void FindNodeWithId( TTreeNode parent, uint32 id, TTreeNode& good );

    TPaneInfoArray  paneInfoArray;
    TTreeWindow*    tree;
    TStatic*        placement;

  DECLARE_RESPONSE_TABLE(TPropTreePage);
};    //{{TPropTreePage}}


// ==========================================================
// TPropTreePane - base class for panes of a TPropTreeDialog
//
inline TTreeNode
TPropTreePage::GetRoot()
{
  return tree->GetRoot();
}

inline TTreeNode
TPropTreePage::GetSelection()
{
  return tree->GetSelection();
}


inline TTreeNode
TPropTreePage::GetFirstVisible()
{
  return tree->GetFirstVisible();
}


inline uint
TPropTreePage::GetIndent()
{
  return tree->GetIndent();
}


inline void
TPropTreePage::SetIndent(uint indent)
{
  tree->SetIndent( indent );
}


inline HIMAGELIST
TPropTreePage::GetImageList(TImageListType type)
{
  return tree->GetImageList( type );
}


inline HIMAGELIST
TPropTreePage::SetImageList(TImageListType type, HIMAGELIST newList)
{
  return tree->SetImageList( type, newList );
}


inline uint
TPropTreePage::GetPaneCount()
{
  return tree->GetItemCount();
}


inline uint
TPropTreePage::GetVisibleCount()
{
  return tree->GetVisibleCount();
}


inline HWND
TPropTreePage::GetEditControl()
{
  return tree->GetEditControl();
}


inline void
TPropTreePage::Update()
{
  tree->Update();
}


inline HTREEITEM
TPropTreePage::HitTest(TTwHitTestInfo* info)
{
}

#include "proptreepane.h"

#endif  // proptreepage_h sentry.

