#include <owl/pch.h>

#include "wizproptreepane.h"
#include "radiowizpane.h"
#include "proptreeapp.rh"

/******************************************************************

  TRadioWizPane - Implementaion of a pane that asks the user a
                  yes or no question and acts accordingly.  Use
                  this class if you want to implement this
                  functionality.  This class can easily be
                  extended to support more than two choices,
                  though.
                    In general, you'll either want to use this
                  class as-is or derive your own class to handle
                  a number of choices.

                  NB:  Make sure the dialog template you use with
                       with this class has two radio buttons
                       with id's IDC_YES and IDC_NO (respectively).
                       Very evil things will happen otherwise...

******************************************************************/


/*
  Constructor for TRadioWizPane

  Purpose: Create the button objects and initialize class memeber variables

  Parameters:
    parent : A pointer to our TWizPropTreeDialog parent
    uniqueId : the unique id of this pane
    ifYes : the id of the pane we go to if the user chooses "yes"
    ifNo : the id of the pane we go to if the user chooses "no"
    theChildren : a TArrayAsVector that holds TChildInfo structures.
                  These children will be created upon the user choosing
                  the "yes" radio button and removed from the tree
                  if "no" is selected
    helpText : a pointer to the help text displayed for this pane. Can be 0
    resId : the resource id of this panes dialog box template
    module : a pointer to the TModule that holds the dialog box template.
             This is usually null.
*/
TRadioWizPane::TRadioWizPane( TWizPropTreeDialog* parent,
                              uint32 uniqueId,
                              uint32 ifYes,
                              uint32 ifNo,
                              TChildInfoArray theChildren,
                              const char far *helpText,
                              TResId resId,
                              TModule* module )
:
  TWizPropTreePane( parent, uniqueId, helpText, resId, false, module ),
  yesNext( ifYes ), noNext( ifNo ), children( theChildren )
{
  yesRadio = new TRadioButton( this, IDC_YESRADIO );
  noRadio  = new TRadioButton( this, IDC_NORADIO );
  panesAdded = false;
}



/*
  TRadioWizPane destructor

  Purpose : Cleans up after this class
*/
TRadioWizPane::~TRadioWizPane()
{
}



/*
  SetupWindow

  Purpose: Call base class's SetupWindow and then check the "Yes" box.
  Pre: none
  Post: All children are Create()'d, yesRadio is checked
*/
void
TRadioWizPane::SetupWindow()
{
  TWizPropTreePane::SetupWindow();

  yesRadio->SetCheck( BF_CHECKED );
}



/*
  KillActive

  Purpose: When we ok KillActive (by returning false), we first
           call SetState( wuDone ) to inform the proptree that
           we're done.  Then if the user has chosen "yes" add our
           child nodes to the tree.  If they chose "no" and we
           have already added our child nodes, remove them.
  Pre: none
  Post: This pane is deactivated

  Return value: Return "true" to disallow the change, "false" to give up
                focus.
*/
bool
TRadioWizPane::KillActive()
{
  //we're done now
  //
  SetWuStatus(wuDone);

  // What's the user want to do?
  //
  if( yesRadio->GetCheck() == BF_CHECKED )
  {
    SetTo( yesNext );

    // If we haven't yet added our panes, do so
    //
    if(!panesAdded)
    {
      panesAdded = true;


      // Loop thru all the kids we need to add...
      //
      TWizPropTreeDialog *wiz = GetWizPropTree();
      TWizPropTreePane *newChild;
      TChildInfoArrayIter iter( children );

      while(iter)
      {

        TChildPaneInfo info = iter.Current();
        newChild = new TWizPropTreePane( wiz,
                                         info.GetId(),
                                         info.GetHelpText().c_str(),
                                         info.GetResId(),
                                         0 );
        wiz->AddChildItem( GetId(),
                           newChild,
                           info.GetTitle().c_str() );
        iter++;
      }
    }
  }
  else //"No" was selected
  {
    SetTo( noNext );

    // If we've added the panes, we need to take them back out.
    //
    if( panesAdded )
    {
      panesAdded = false;

      TWizPropTreeDialog *wiz = GetWizPropTree();
      TChildInfoArrayIter iter( children );

      while(iter)
      {
        wiz->RemoveChildItem( iter.Current().GetId() );
        iter++;
      }
    }
  }

  return false;
}


