/*
  WIZPROPTREE.CPP
  ---------------
  TWizPropTreeDialog - implementation

  Purpose:  Implementation of wizard functionality using a property
            tree.  NB.  This class requires dummy buttons from
            TPropTreeDialog - just place them somewhere on the dialog
            and set them all to ~WS_VISIBLE so that they don't get in
            the way.  In the next release of TPropTreeDialog we won't
            need to do this, but until then...

   Author: Russell Morris
   Release Date:
*/


#include <owl/pch.h>

#include "wizproptree.h"
#include "wizproptreepane.h"
#include "proptreeapp.rh"

// Our response table
//
DEFINE_RESPONSE_TABLE1( TWizPropTreeDialog, TPropTreeDialog )
  EV_BN_CLICKED( IDC_NEXT, CmNext ),
  EV_BN_CLICKED( IDC_PREVIOUS, CmPrevious ),
  EV_BN_CLICKED( IDHELP, CmHelp ),
END_RESPONSE_TABLE;


/*
 Constructor for TWizPropTreeDialog
 Purpose: Sets up the check image list, initializes button states
          and creates button objects.
 Pre: none
 Post: This class instance is ready to go.

 Parameters:
   parent : a pointer to the parent of this dialog.
   wizData : a pointer to the TWizData derived class for this wizard.
   resId : a resource identifier that specifies the proptree's dialog
           template.
   module : (optional) specifies a pointer to the TModule that contains
            the dialog template with resource identifier resId.
*/
TWizPropTreeDialog::TWizPropTreeDialog( TWindow *parent,
                                        TWizData *wizData,
                                        TResId resId,
                                        TModule *module )
:
  TPropTreeDialog( parent, resId, module )
{
  // Our TWizData...
  //
  data = wizData;

  //Create Image list
  wizBitmaps = new TImageList( *GetApplication(),
                                TResId(IDB_WIZARDCHECKS),
                                11,
                                0,
                                TColor::LtYellow,
                                IMAGE_BITMAP,
                                0);
  //Set these to true
  Next = Cancel = true;

  // Create our buttons
  //
  NextButton = new TButton( this, IDC_NEXT );
  PreviousButton = new TButton( this, IDC_PREVIOUS );
  CancelButton = new TButton( this, IDCANCEL );
}



/*
  Destructor for TWizPropTreeDialog
  Purpose: cleans up after an instance of TWizPropTreeDialog
  Pre: none
  Post: All of the memory and resources used by this class instance are
        freed
*/
TWizPropTreeDialog::~TWizPropTreeDialog()
{
  Destroy();
  delete wizBitmaps;
}



/*
    SetupWindow
    Purpose:  Does setup right before a window is displayed.  Calls the
              base class SetupWindow(), then sets the image list in the
              tree and initializes the button states.
    Pre: Window has just been created
    Post: Window is ready to be displayed.
*/
void
TWizPropTreeDialog::SetupWindow()
{
  TPropTreeDialog::SetupWindow();

  SetImageList( TTreeWindow::Normal, *wizBitmaps );

  EnableNext( true );
  EnablePrevious( false );
}


/*
  NextToFinish
  Purpose:  Changes the "Next" button to say "Finish" if toFinish is true.
            Otherwise, changes text back to "Next >".
  Pre: none
  Post: If toFinish is true, the "Next >" button's text is changed to
        "Finish", and vice versa.

  Parameters:
    bool toFinish : parameter specifying what action to take - ie change
                    "Next >" to "Finish" or vice versa.
*/
void
TWizPropTreeDialog::NextToFinish( bool toFinish )
{
  Next = !toFinish;
  (toFinish) ? NextButton->SetWindowText("Finish") :
               NextButton->SetWindowText("Next >");
}


/*
  NextIsFinish
  Purpose:  Determines whether the "Next >" button says "Next >" or
            "Finish".
  Pre: none
  Post: see return value below.

  Return Value:  Returns true if the Next button's text is "Next >", false
                 if the text is "Finish".
*/
bool
TWizPropTreeDialog::NextIsFinish()
{
  return !Next;
}


/*
 CancelToClose
 Purpose:  Changes the Cancel button's text to "Close" if toClose is true,
           otherwise it sets the text to Cancel.  Updates Cancel boolean
           member.  Use this once you have reached a "point of no return"
           in your processing - ie the user can't undo changes that have
           been made.
 Pre: none
 Post:  The text of the "Cancel" button is updated depending on toClose.
        The Cancel data member is set to !toClose.

 Parameters:
   toClose: Specifies whether to change the text to "Cancel" or "Close".
*/
void
TWizPropTreeDialog::CancelToClose( bool toClose )
{
  Cancel = !toClose;
  (toClose) ? CancelButton->SetWindowText("Close") :
              CancelButton->SetWindowText("Cancel");
}


/*
 CancelIsClose
 Purpose: Determines whether the Cancel button says "Cancel" or "Close".
 Pre: none
 Post: see return value below

 Return value: Returns true if the Cancel button currently displays the
               text "Close" instead of cancel.
*/
bool
TWizPropTreeDialog::CancelIsClose()
{
  return !Cancel;
}


/*
 EnableNext
 Purpose: Enables the Next button depending on the value of bEnable
 Pre: none
 Post: Enables the "Next >" button depeding on the value of bEnable.

 Parameter:
   bEnable : detmines the status of the Next button
             false = disabled
             true = enabled
*/
void
TWizPropTreeDialog::EnableNext( bool bEnable )
{
  NextButton->EnableWindow( bEnable );
}


/*
  EnablePrevious
  Purpose: Enable / Disable the "Previous" button.  Use this when you do
           not want the user to backtrack.
  Pre: none
  Post: Enables the Previous button depending on the value of bEnable.

  Parameter:
    bEnable : if true, enables the button, disables if false.
*/
void
TWizPropTreeDialog::EnablePrevious( bool bEnable )
{
  PreviousButton->EnableWindow( bEnable );
}


/*
 CmNext
 Purpose: Asks the pane if it's ok to leave and go to the next pane.
          If it is, it asks which pane is next and then selects that
          pane via it's unique id.  If the "Next" button says "Finish",
          simply calls CloseWindow() to kill the dialog.
 Pre: Called in response to the press of the "Next" button
 Post: The next pane is selected, or the dialog is closed altogether.

*/
void
TWizPropTreeDialog::CmNext()
{
  TWizPropTreePane *thisPane = GetCurrentWizPane();

  // if it's ok to leave this step...
  // BTW:  the pane is responsible for bitching if KillActive returns false
  if( thisPane && !thisPane->KillActive() )
  {
    // If the "Next >" button says "Finish", we need to shut down...
    //
    if( NextIsFinish() )
      CloseWindow(IDOK);
    else
    {
      // Find out where to go next and go there...
      uint32 nextPane = thisPane->WhereToNext();

      // If nextPane == -1, then this pane is the last pane in the list.
      // It should have disabled the Next button, but obviously it didn't...
      //
      if( nextPane != (uint32)-1 )
        SelectPane( nextPane );
    }
  }
}


/*
 CmPrevious
 Purpose : When "Previous" is pressed, we are not concerned with what
           pane was active right before us.  We are concerned about
           the sibling node previous to us.  This way, we step back
           to the previous step on our level - we dont' want to return
           to a step lower than we are at.  If we are the first step
           in a process, return to the main step in that process
           (ie our parent node).
 Pre: Called in response to the user pressing the "Previous" button.
 Post: Our previous sibling (or our parent if none exists) is visited.
*/
void
TWizPropTreeDialog::CmPrevious()
{
  TTreeNode thisNode = GetPaneNode( GetCurrentWizPane()->GetId() );

  if( thisNode.GetPrevSibling() )
    SelectNextPane( TTreeNode::Previous );

  else //There's no one in front of us, we need to go back to our parent.
  SelectNextPane( TTreeNode::Parent );

  // And make sure that "Next" says "Next", not "Finish"
  //
  NextToFinish(false);
}


/*
  CmHelp
  Purpose:  Virtual help function - override for functionality in other
            classes.
  Pre: Called in response to the "Help" button being pressed.
  Post: Help is performed

  Comments:  You'll probably want to find out which pane is active, and
             then bring up the relevant help topic.
*/
void
TWizPropTreeDialog::CmHelp()
{
}


/*
 WizPaneUpdated
 Purpose: Redraw the nodes if there has been a change in their state.
          Since we only need to redraw the nodes in the heirarchy of
          the node that sent us this message, just call UpdateNode()
          on it.
 Pre: "id" must be a valid unique id of a pane in the tree.
 Post: The pane's entire heirarchy (itself on up the tree) is updated
       to reflect the new status of the pane with id "id"

 Parameters:
   id : a valid unique identifier of a pane in the tree

 Comment:  Since TWizPropTreeDialog takes care of doing this sort of stuff
           when a pane is left, you usually don't need to call this.  The
           only time you would call this is if you wanted to let a user
           leave a pane while something is still not finished (but that
           something is guarunteed to finish - like maybe a large
           calculation or an network transfer of some sort...).  When your
           pane realizes it's done, it should call this function so that
           the whole node heirarchy is updated.

*/
void
TWizPropTreeDialog::WizPaneUpdated( uint32 id )
{
  TTreeNode node = GetPaneNode( id );

  UpdateNode( node );
}


/*
  UpdateNode
  Purpose: UpdateNode updates a nodes entire heirarchy.  It finds the
           node's highest ancestor (ie a child of the root) and starts
           the process from there.  This way, all the changes that this
           node may affect will be realized.
  Pre: "node" must be a valid node in the tree.
  Post: "node"'s entire heirarchy is updated to reflect the latest status
         of all its children.
*/
void
TWizPropTreeDialog::UpdateNode( TTreeNode& node )
{
  TWizardUpdate wu;

  // Walk all the way back up the tree and find our ancestor who has
  // TVI_ROOT for a parent.
  //
  TTreeNode highest = node;
  while( (HTREEITEM)highest.GetParent() )
  {
    highest = highest.GetParent();
  }

  wu = GetNodeStatus( highest );

  // The TWizardUpdate enum is designed to be an index into the image list
  //
  highest.SetImageIndex( wu );

  //  If this node is done, collapse it.
  //
  if( wu == wuDone )
  {
    highest.ExpandItem( TTreeNode::Collapse );
  }
}


/*
 UpdateNodeHeirarchy
 Purpose:  Goes thru each top level node (children of TVI_ROOT) and
           updates their heirarchy by calling UpdateNode on each of them.
 Pre: none
 Post: all nodes in the tree are updated.

*/
void
TWizPropTreeDialog::UpdateNodeHeirarchy()
{
  TTreeNode root(*GetTree(), TVI_ROOT);

  // Update each of the top level nodes
  //
  for( TTreeNode child = root.GetChild();
       (HTREEITEM)child;
       child = child.GetNextSibling() )
  {
    UpdateNode(child);
  }
}



/*
  GetNodeStatus
  Purpose:  Determine the status of a given node by looking at the wu
            status of all its kids and grandkids.
  Pre: node must be an existing node in the tree
  Post: The node's wu status is determined and returned

  Parameters:
    node : a valid TTreeNode which exists in the tree

  Return value:
    Returns the wu status of "node"
*/
TWizardUpdate
TWizPropTreeDialog::GetNodeStatus( TTreeNode& node )
{
  TWizardUpdate wu;
  TWizPropTreePane *pane;

  // Get a pointer to the pane
  //
  node.GetItemData( (uint32)pane );
  if(!pane)
    return wuNonDeterminate;

  // Get our personal wu status
  //
  wu = pane->GetWizPaneStatus();

  // And OR it in with each of our kids
  //
  for( TTreeNode child = node.GetChild();
       (HTREEITEM)child;
       child = child.GetNextSibling() )
  {
    // Make a recursive call to our kid to get it's wu status.  It will
    // determine its status by checking all its kids also...
    //
    wu |= GetNodeStatus( child );
  }

  // The TWizardUpdate enum is designed to be an index into the image list
  //
  node.SetImageIndex( (int)wu );
  node.SetSelectedImageIndex( wuSelected );

  return wu;
}

/*
 GetWizPane
 Purpose: Cast it in advance so we don't have to clutter up code
 Pre: id must be the unique id of a pane in the tree
 Post: The TPropTreePane* pointer is cast to TWizPropTreePane

 Parameters:
   id : a unique id of a TWizPropTree pane in the tree

 Return value:
   Returns the TPropTreePane* cast to TWizPropTreePane*
*/
TWizPropTreePane*
TWizPropTreeDialog::GetWizPane( uint32 id )
{
  return dynamic_cast<TWizPropTreePane*>( TPropTreeDialog::GetPane(id) );
}


/*
 GetCurrentWizPane
 Purpose: Cast it in advance so we don't have to clutter up code
 Pre: A pane must be selected.
 Post: The TPropTreePane* pointer is cast to TWizPropTreePane

 Return value:
   Returns the TPropTreePane* cast to TWizPropTreePane*
*/
TWizPropTreePane*
TWizPropTreeDialog::GetCurrentWizPane()
{
  return
    dynamic_cast<TWizPropTreePane*>( TPropTreeDialog::GetCurrentPane() );
}



/*
 AddPane
 Purpose: Add panes with the I_IMAGECALLBACK so that we can set each node
          to a bitmap displaying its current state.
 Pre: pane must be a valid pointer to a TWizPropTreePane.  text can be null,
      in which case the title of "pane" will be used.
 Post: The new pane is added to the tree.

 Parameters:
   pane : a valid pointer to a TWizPropTreePane
   parent : a TTreeNode which will serve as the parent node for this new
            pane.
   text : A pointer to the text that is to be displayed in the pane's tree
          node.  If this is null, the pane's dialog title will be used
          instead.

 Return value:
   Once this node has been added to the tree, we return that node represeting
   "pane" in the tree so that it may be used elsewhere by the calling
   function.
*/
TTreeNode
TWizPropTreeDialog::AddPane( TWizPropTreePane* pane,
                             TTreeNode parent,
                             const char far* text )
{
  return TPropTreeDialog::AddPane( pane, parent, text,
                                   wuNotDone,
                                   wuSelected );
}



/*
  PreSwitchPanes

  Purpose: We don't want the user to be able to use the mouse to select
           panes - we need them to use the Next and Previous buttons
           instead so we can completely control the flow of the wizard.
  Pre: TWizPropTree has just recieved the EvSecChanging notification
       and we are the first it asks permission to change.
  Post: We'll return true to stop the panes from changing if the user
        has used the mouse to select the new pane

  Parameters:
    nm : A reference to a TTwNotify

  Return value: returns false to allow panes to change, true otherwise
*/
bool
TWizPropTreeDialog::PreSwitchPanes( TTwNotify& nm )
{
  // We don't want the user to select a node by the keyboard or mouse -
  // we want to be in complete control of node selection.
  //
  return (nm.action == TVC_BYMOUSE) || (nm.action == TVC_BYKEYBOARD);
}


/*
  PanesSwitched
  Purpose: Allows updating of things when two panes are switched in the
           tree.  This function is virtual and is a redefinition of
           TPropTreeDialog's PanesSwitched().
  Pre: oldPane and newPane can both be either null or valid pointers
  Post: none

  Parameters:
    oldPane : a pointer to the pane that was just left.  Might be nil.
    newPane : a pointer to the pane that was just selected.  Might be null,
              but probably not unless something weird is going on.
*/
void
TWizPropTreeDialog::PanesSwitched( TPropTreePane *oldPane,
                                   TPropTreePane *newPane  )
{
  //  Make the pretty casts...
  //
  TWizPropTreePane *oldWizPane =
    dynamic_cast<TWizPropTreePane*>(oldPane);
  TWizPropTreePane *newWizPane =
    dynamic_cast<TWizPropTreePane*>(newPane);

  TTreeNode node(*GetTree(), TVI_ROOT);
  if(oldWizPane)
  {
    node = GetPaneNode( oldWizPane->GetId() );
    if( !node )
      return;

    // Make sure everything looks nice
    //
    UpdateNode(node);

  }

  // If this is the first pane, disable the previous button.
  //
  if( newWizPane && newWizPane->GetId() == 0 )
    EnablePrevious( false );
  else
    EnablePrevious();
}


/*
  AddChildItem
  Purpose: We just want to override this one so that we can add child items without
           specifying image indexes.
  Pre: parentId must be a valid identifier of a pane in the tree
  Post: newPane is added to the tree as a child of the pane with "parentId".

  Parameters:
    parentId : a unique identifier of a pane in the tree
    newPane : a pointer to a TWizPropTreePane* that will be added
    title : a pointer to text that will be used in the pane's node.  This
            can be null, in which case the pane's title will be used.
*/
void
TWizPropTreeDialog::AddChildItem( uint32 parentId,
                                  TWizPropTreePane *newPane,
                                  const char far *title )
{
  // Just call the base class function
  //
  TPropTreeDialog::AddChildItem( parentId,
                                 newPane,
                                 title,
                                 wuNotDone,
                                 wuSelected );
  // And update the node
  //
  TTreeNode parent = GetPaneNode( parentId );
  if( parent )
    UpdateNode( parent );
}

/*
  RemoveChildItem
  Purpose: When removing a child item, we need to update its parents to
           reflect the changes.
  Pre: "removeId" must be a valid identifier of a pane in the tree.
  Post: the node with id "removeId" is removed from the tree.  The pane
        object is not destroyed yet - it will be killed when we close.

  Parameter:
    removeId : a unique identifier of a pane in the tree.
*/
void
TWizPropTreeDialog::RemoveChildItem( uint32 removeId )
{
  // First let's find out who is the parent
  //
  TTreeNode parent = GetPaneNode( removeId ).GetParent();

  // Call the function we override so that it can do the dirty work.
  //
  TPropTreeDialog::RemoveChildItem( removeId );

  // And lastly we need to update the tree so we can see the effects of
  // removing the node
  //
  if( parent )
    UpdateNode(parent);
}


/*
  GetWizData

  Purpose: Allow panes to get at our TWizData data structure.
  Pre: none
  Post: none

  Return value: returns a pointer to our TWizData structure
*/
TWizData*
TWizPropTreeDialog::GetWizData()
{
  return data;
}



