#ifndef __WIZPROPTREEDIALOG_H
#define __WIZPROPTREEDIALOG_H

/*********************************************************
  TWizPropTreeDialog - implement wizard functionality.

**********************************************************/
#include <owl/imagelst.h>
#include <owl/button.h>

#include "proptree.h"

/*********************************************************************

  TWizData - Encapsulation of the wizard process.
               Since the entire point of using a wizard is to get a
             large chunk of info step by step, it's logical to have
             a single class that can keep track of where we are in
             the process (how much data we have, what we need, etc...).
               TWizData is completely abstract - you will need to
             derive your own class from it.  This new class should be
             able to hold all the info that you can expect to get from
             the user during the wizard process.
               TWizPropTreeDialog requires a pointer to this TWizData
             class, which it makes available to all panes via a call
             to GetWizData().  Of course, in your panes you will probably
             want to redefine this function so that you don't have to
             dynamic_cast it all the time.

*********************************************************************/
class TWizData {
  public:

    // Since TWizData does essentially nothing, you'll need to redefine
    // it with meaningfull parameters and stuff.
    //
    TWizData()  {}
    ~TWizData() {}
};


class TWizPropTreePane;


    // enum the states that a wizard pane might be in...  determines
    // whether the pane (and it's parent) display a check, empty box,
    // a blue X, or a grayed box.
    //
    //  wuDone:  Means that the step the pane represents - as well as
    //           all of it's child panes - has been completed.  The
    //           pane's node displays a check.
    //
    //  wuNotDone: Means that the step the pane represents has not
    //             not been completed none of it's child panes have
    //             been completed.  The pane's node displays an
    //             empty box.
    //
    //  wuNotAllDone:  Means that some of the parent's nodes are
    //                 wuDone and some are wuNotDone.  Displays a
    //                 grayed-out check.
    //
    //  wuNonDeterminate: Means that the step that the pane represents
    //                    does not necessarily have to be completed in
    //                    order for the Wizard to finish.  Displays a
    //                    blue X.
    //
    //  wuSelected  This node is currently selected.  No functions return
    //              this value - it is used to set the selected image index.
    //
    // These states are used to index the image list.
    //
    typedef uint TWizardUpdate;
    const TWizardUpdate wuNonDeterminate = 0x0000;
    const TWizardUpdate wuDone           = 0x0001;
    const TWizardUpdate wuNotDone        = 0x0002;
    const TWizardUpdate wuNotAllDone     = 0x0003;
    const TWizardUpdate wuSelected       = 0x0004;

class TWizPropTreeDialog : public TPropTreeDialog {

  public:
    TWizPropTreeDialog( TWindow* parent,
                        TWizData* wizData,
                        TResId resId,
                        TModule* module = 0 );

    virtual ~TWizPropTreeDialog();


    // Panes call this when they change.  It makes sure to update
    // the nodes in the tree so that they are displayed correctly.
    //
    void WizPaneUpdated( uint32 id );

    TWizPropTreePane* GetWizPane( uint32 id );
    TWizPropTreePane* GetCurrentWizPane();

    virtual void SetupWindow();

    // Add a pane to the tree - we don't allow nodes to have images.
    //
    TTreeNode AddPane( TWizPropTreePane* pane,
                       TTreeNode parent,
                       const char far* text );

    // Set the text of certain buttons.
    //
    void NextToFinish( bool toFinish = true );
    bool NextIsFinish();
    void CancelToClose( bool toClose = true );
    bool CancelIsClose();

    // Disable/enable buttons
    //
    void EnableNext( bool bEnable = true );
    void EnablePrevious( bool bEnable = true );

    // Allow panes to add child items to themselves.  For example, if you
    // ask the user if they want to do something, you may want to add
    // different panes depending on whether they choose "yes" or "no".
    // Don't add child items unless you know that they will be used.
    // If the user changes their mind, you can remove child items by
    // calling RemoveChildItem();
    //
    void AddChildItem( uint32 paneId,
                       TWizPropTreePane* newChildPane,
                       const char far* title );
    void RemoveChildItem( uint32 childToRemoveId );

    // This function allows panes to get at the TWizData we keep
    //
    TWizData *GetWizData();

  protected:


    // Button press response functions
    //
    void CmNext();
    void CmPrevious();
    void CmHelp();

    // We don't want the user to be able to select panes with the mouse
    //
    virtual bool PreSwitchPanes( TTwNotify& nm );
    
    // We need to update the state of each pane as it is left.
    //
    virtual void PanesSwitched( TPropTreePane* oldPane,
                                TPropTreePane* newPane );

  private:

    // Internal functions that determine what type of bitmap to display
    // for a wizard pane
    //
    void UpdateNodeHeirarchy();
    void UpdateNode(TTreeNode& node);
    TWizardUpdate GetNodeStatus( TTreeNode& node );

    // Pointer to the wizard bitmaps
    //
    TImageList *wizBitmaps;

    // Bools to tell us whether the Next button says "Next >" or "Finish"
    // or the Cancel button says "Cancel" or "Close"
    //
    bool Next;
    bool Cancel;

    // TButton objects to let us get at the Next, Previous, etc buttons
    // to change their text around and stuff...
    //
    TButton *NextButton;
    TButton *PreviousButton;
    TButton *CancelButton;

    // The data we have obtained thus far in the wizard process
    //
    TWizData *data;

    DECLARE_RESPONSE_TABLE(TWizPropTreeDialog);
};





#endif
