/*
  TWizPropTreePane - implementation of wizard functionality for
                     TPropTreePanes.  This class is used as the base class
                     for any panes added to a TWizPropTreeDialog.

  Author: Russell Morris
  Release Date: February 26, 1997
*/
#include <owl/pch.h>

#include "proptreepane.h"
#include "wizproptreepane.h"


/*
  Constructor for TWizPropTreePane
  Purpose: Sets up data members and other fields based on parameters passed
           to it
  Pre:  "parent" is not null, uniqueId > 0, "resId" should be a resource
        identifier of a WS_CHILD and WS_CONTROLPARENT dialog template,
        "module" (optionally null)_should point to the TModule that contains
        the dialog template with id "resId".
  Post: the object instance is all set up and ready to use

  Parameters:
    parent : A valid pointer to a TWizPropTreeDialog which is the parent
             of this pane.
    uiniqueId : A distinct (not other pane has the same one) id number
                that this pane will be identified by.  It is best to have
                the first pane in the tree be 0, and start numbering
                the rest of the pane's ids from there.
    helpText : The text to be displayed in the little pane info box
               under the tree.
    resId : A resource identifier of the dialog box template for this
            pane.
    module : Usually NULL, but if not it points to the TModule that has
             the dialog box template identified by resId.

*/
TWizPropTreePane::TWizPropTreePane( TWizPropTreeDialog* parent,
                                    uint32 uniqueId,
                                    const char far *helpText,
                                    TResId resId,
                                    bool isLast,
                                    TModule* module )
:
    TPropTreePane(parent, uniqueId, helpText, resId, module)
{
  // initialize from and to.
  from = to = -1;

  // Are we the last pane in the tree?
  //
  last = isLast;

  SetWuStatus(wuNotDone);
}


/*
  Destructor for TWizPropTreePane

  Purpose: Cleans up after this instance of the class.  Calls Destroy() to
           kill the window element.
  Pre: called when class is deleted
  Post: all memory and stuff used by class is given back to system
*/
TWizPropTreePane::~TWizPropTreePane()
{
  Destroy();
}


/*
  SetupWindow

  Purpose: We need to do some default behaviour and set where we lead
           to when the "Next >" button is pressed.
  Pre: virtual function called by TWindow
  Post: Window is setup and ready to be displayed
*/
void
TWizPropTreePane::SetupWindow()
{
  TPropTreePane::SetupWindow();

  // This default behaviour assumes that your id's follow the pattern
  // this pane's id = (1 + the previous pane's id).
  //
  SetTo( GetId() + 1 );
}


/*
  SetActive

  Purpose:  Called to request our permission to recieve focus.  Our default
            implementation here is to grant that request.  If we are the
            last pane in the list, then we need to turn the "Next >" button
            into "Finish".
  Pre: We are about to recieve focus
  Post: see return value below

  Return value:  Returns "true" to refuse to accept focus, "false" to
                 accept focus.
*/
bool
TWizPropTreePane::SetActive()
{
  // If we're the last pane in the list, call NextToFinish()
  //
  if(last)
    GetWizPropTree()->NextToFinish( true );
  else
    GetWizPropTree()->NextToFinish( false );

  return false;
}



/*
  GetWizPaneStatus

  Purpose: Return the status of our pane.  This is called when our
           TWizPropTree wants to know how we're doing.  We only return
           our status - we don't worry about our children.
  Pre: none
  Post: none

  Return value: a TWizardUpdate constant (typedef'd as uint) specifying
                the current status of this pane.  You will almost
                never need to redefine this function.
*/
TWizardUpdate
TWizPropTreePane::GetWizPaneStatus()
{
  return GetWuStatus();
}


/*
  WizPaneStatusChange

  Purpose:  This function informs the parent TWizPropTreeDialog that this
            pane's status has changed.  The TWizPropTreeDialog will take
            care of redrawing node bitmaps of this pane's node's parents.
  Pre: none
  Post: The tree updates the nodes in the tree to reflect the change in
        status of this pane.  See TWizPropTreeDialog::WizPaneUpdated() for
        more details

  Comment:  This shouldn't normally be called.  If you need to
*/
void
TWizPropTreePane::WizPaneStatusChange()
{
  TWizPropTreeDialog *parent = GetWizPropTree();
  if(!parent)
    return;
  parent->WizPaneUpdated( GetId() );
}


/*
  WhereToNext

  Purpose: Returns where this pane leads to next.
  Pre: none
  Post: see return value below

  Return value: Returns the uiniqueId of the pane that is supposed to
                come right after us.
*/
uint32
TWizPropTreePane::WhereToNext()
{
  return to;
}


/*
  Purpose: By default, we'll have KillActive() determine that we are all
           done and return "false" to indicate so.  This is so you can
           use this class for panes that don't require any input (splash
           panes).
  Pre: Called by TWizPropTreeDialog when it wants our permission
       to give another pane the focus.
  Post: see return value below

  Return value: Returns "true" if we don't want to give up the focus,
                "false" to give up focus.

  Comments:  If you return "true" to keep the focus, it is your
             responsibility to bitch at the user and tell them what
             is wrong.
*/
bool
TWizPropTreePane::KillActive()
{
  SetWuStatus( wuDone );
  return TPropTreePane::KillActive();
}



/*
  SetWuStatus

  Purpose: Sets the wuStatus of this pane.  This is a protected pane
           function for use be derived classes.  When you need to update
           the status of your pane, call this function.
  Pre: wuStatus is a valud TWizardUpdate status
  Post: the pane's status data member is updated

  Parameters:
    wu : The new status for this pane.  Must be a valid wuXxx constant.
*/
void
TWizPropTreePane::SetWuStatus( TWizardUpdate wu )
{
  status = wu;
}


/*
  GetWizData

  Purpose: Wraps the call to TWizPropTreeDialog->GetWizData
  Pre: none
  Post: none

  Return value: Returns a pointer to the TWizData for our
                TWizPropTreeDialog

  Comments:  You'll almost certainly want to redefine this function in each
             of your TWizPropTreePane descendant classes so that you don't
             have to always dynamic_cast.
*/
TWizData*
TWizPropTreePane::GetWizData()
{
  return GetWizPropTree()->GetWizData();
}






